#include "stdafx.h"
#include "Mesh/Material.h"

using namespace Egg;

Mesh::Material::Material(
	Microsoft::WRL::ComPtr<ID3D11VertexShader		>	vertexShader		,
	Microsoft::WRL::ComPtr<ID3D11HullShader			>	hullShader			,
	Microsoft::WRL::ComPtr<ID3D11DomainShader		>	domainShader		,
	Microsoft::WRL::ComPtr<ID3D11GeometryShader		>	geometryShader		,
	Microsoft::WRL::ComPtr<ID3D11RasterizerState	>	rasterizerState		,
	Microsoft::WRL::ComPtr<ID3D11PixelShader		>	pixelShader			,
	Microsoft::WRL::ComPtr<ID3D11DepthStencilState	>	depthStencilState	,
	Microsoft::WRL::ComPtr<ID3D11BlendState			>	blendState			,
	Microsoft::WRL::ComPtr<ID3D11ComputeShader		>	computeShader		):
	vertexShader		( vertexShader		),
	hullShader			( hullShader		),	
	domainShader		( domainShader		),
	geometryShader		( geometryShader	),	
	rasterizerState		( rasterizerState	),	
	pixelShader			( pixelShader		),	
	depthStencilState	( depthStencilState	),
	blendState			( blendState		),	
	computeShader		( computeShader		)
{}

Mesh::Material::~Material()
{
}

void Mesh::Material::apply(Microsoft::WRL::ComPtr<ID3D11DeviceContext> context)
{
	context->VSSetShader(vertexShader	.Get(), nullptr, 0);
	context->HSSetShader(hullShader		.Get(), nullptr, 0);
	context->DSSetShader(domainShader	.Get(), nullptr, 0);
	context->GSSetShader(geometryShader	.Get(), nullptr, 0);
	context->RSSetState( rasterizerState.Get());
	context->PSSetShader(pixelShader	.Get(), nullptr, 0);
	context->OMSetDepthStencilState( depthStencilState.Get(), 0);
	float ones[] = { 1, 1, 1, 1 };
	context->OMSetBlendState(blendState.Get(), ones, 0xff);
	context->CSSetShader(computeShader.Get(), nullptr, 0);

	ID3D11Buffer* buffers[64];
	uint i = 0;
	for (auto& cb : vsCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->VSSetConstantBuffers(0, vsCbs.size(), buffers);

	i = 0;
	for (auto& cb : hsCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->HSSetConstantBuffers(0, hsCbs.size(), buffers);

	i = 0;
	for (auto& cb : dsCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->DSSetConstantBuffers(0, dsCbs.size(), buffers);

	i = 0;
	for (auto& cb : gsCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->GSSetConstantBuffers(0, gsCbs.size(), buffers);

	i = 0;
	for (auto& cb : psCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->PSSetConstantBuffers(0, psCbs.size(), buffers);

	i = 0;
	for (auto& cb : csCbs)
	{
		buffers[i++] = cb.Get();
	}
	context->CSSetConstantBuffers(0, csCbs.size(), buffers);

	ID3D11SamplerState* samplers[64];
	i = 0;
	for (auto& ss : vsSss)
	{
		samplers[i++] = ss.Get();
	}
	context->VSSetSamplers(0, vsSss.size(), samplers);

	i = 0;
	for (auto& ss : hsSss)
	{
		samplers[i++] = ss.Get();
	}
	context->HSSetSamplers(0, hsSss.size(), samplers);

	i = 0;
	for (auto& ss : dsSss)
	{
		samplers[i++] = ss.Get();
	}
	context->DSSetSamplers(0, dsSss.size(), samplers);

	i = 0;
	for (auto& ss : gsSss)
	{
		samplers[i++] = ss.Get();
	}
	context->GSSetSamplers(0, gsSss.size(), samplers);

	i = 0;
	for (auto& ss : psSss)
	{
		samplers[i++] = ss.Get();
	}
	context->PSSetSamplers(0, psSss.size(), samplers);

	i = 0;
	for (auto& ss : csSss)
	{
		samplers[i++] = ss.Get();
	}
	context->CSSetSamplers(0, csSss.size(), samplers);

	ID3D11ShaderResourceView* srvs[64];
	i = 0;
	for (auto& sr : vsSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->VSSetShaderResources(0, i, srvs);

	i = 0;
	for (auto& sr : hsSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->HSSetShaderResources(0, i, srvs);

	i = 0;
	for (auto& sr : dsSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->DSSetShaderResources(0, i, srvs);

	i = 0;
	for (auto& sr : gsSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->GSSetShaderResources(0, i, srvs);

	i = 0;
	for (auto& sr : psSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->PSSetShaderResources(0, i, srvs);

	i = 0;
	for (auto& sr : csSrs)
	{
		srvs[i++] = sr.Get();
	}
	if (i)	context->CSSetShaderResources(0, i, srvs);
}

