#pragma once
#include <map>
#include <vector>
#include "ShaderStageFlag.h"

namespace Egg { namespace Mesh
{
	GG_CLASS(Material)
		Microsoft::WRL::ComPtr<ID3DBlob> inputSignature;

		Microsoft::WRL::ComPtr<ID3D11VertexShader		>	vertexShader;
		Microsoft::WRL::ComPtr<ID3D11HullShader			>	hullShader;
		Microsoft::WRL::ComPtr<ID3D11DomainShader		>	domainShader;
		Microsoft::WRL::ComPtr<ID3D11GeometryShader		>	geometryShader;
		Microsoft::WRL::ComPtr<ID3D11RasterizerState	>	rasterizerState;
		Microsoft::WRL::ComPtr<ID3D11PixelShader		>	pixelShader;
		Microsoft::WRL::ComPtr<ID3D11DepthStencilState	>	depthStencilState;
		Microsoft::WRL::ComPtr<ID3D11BlendState			>	blendState;
		Microsoft::WRL::ComPtr<ID3D11ComputeShader		>	computeShader;

		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	vsCbs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	hsCbs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	dsCbs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	gsCbs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	psCbs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11Buffer	>	>	csCbs;

		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		vsSss;
		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		hsSss;
		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		dsSss;
		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		gsSss;
		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		psSss;
		std::vector<Microsoft::WRL::ComPtr<ID3D11SamplerState	>	>		csSss;

		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		vsSrs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		hsSrs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		dsSrs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		gsSrs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		psSrs;
		std::vector<Microsoft::WRL::ComPtr<ID3D11ShaderResourceView	>	>		csSrs;

	protected:
		Material(
			Microsoft::WRL::ComPtr<ID3D11VertexShader		>	vertexShader,
			Microsoft::WRL::ComPtr<ID3D11HullShader			>	hullShader,
			Microsoft::WRL::ComPtr<ID3D11DomainShader		>	domainShader,
			Microsoft::WRL::ComPtr<ID3D11GeometryShader		>	geometryShader,
			Microsoft::WRL::ComPtr<ID3D11RasterizerState	>	rasterizerState,
			Microsoft::WRL::ComPtr<ID3D11PixelShader		>	pixelShader,
			Microsoft::WRL::ComPtr<ID3D11DepthStencilState	>	depthStencilState,
			Microsoft::WRL::ComPtr<ID3D11BlendState			>	blendState,
			Microsoft::WRL::ComPtr<ID3D11ComputeShader		>	computeShader);
	public:

		~Material();
		
		void setCb(ShaderStageFlag stage, uint index, Microsoft::WRL::ComPtr<ID3D11Buffer> cb)
		{
			switch (stage)
			{
			case ShaderStageFlag::Vertex:
				if (vsCbs.size() <= index) vsCbs.resize(index + 1);	vsCbs.at(index) = cb;	break;
			case ShaderStageFlag::Hull:
				if (hsCbs.size() <= index) hsCbs.resize(index + 1);	hsCbs.at(index) = cb;	break;
			case ShaderStageFlag::Domain:
				if (dsCbs.size() <= index) dsCbs.resize(index + 1);	dsCbs.at(index) = cb;	break;
			case ShaderStageFlag::Geometry:
				if (gsCbs.size() <= index) gsCbs.resize(index + 1);	gsCbs.at(index) = cb;	break;
			case ShaderStageFlag::Pixel:
				if (psCbs.size() <= index) psCbs.resize(index + 1);	psCbs.at(index) = cb;	break;
			case ShaderStageFlag::Compute:
				if (csCbs.size() <= index) csCbs.resize(index + 1);	csCbs.at(index) = cb;	break;
			}
		}
		void setSamplerState(ShaderStageFlag stage, uint index, Microsoft::WRL::ComPtr<ID3D11SamplerState> ss)
		{
			switch (stage)
			{
			case ShaderStageFlag::Vertex:	if (vsSss.size() <= index) vsSss.resize(index + 1);	vsSss.at(index) = ss;	break;
			case ShaderStageFlag::Hull:		if (hsSss.size() <= index) hsSss.resize(index + 1);	hsSss.at(index) = ss;	break;
			case ShaderStageFlag::Domain:	if (dsSss.size() <= index) dsSss.resize(index + 1);	dsSss.at(index) = ss;	break;
			case ShaderStageFlag::Geometry:	if (gsSss.size() <= index) gsSss.resize(index + 1);	gsSss.at(index) = ss;	break;
			case ShaderStageFlag::Pixel:	if (psSss.size() <= index) psSss.resize(index + 1);	psSss.at(index) = ss;	break;
			case ShaderStageFlag::Compute:	if (csSss.size() <= index) csSss.resize(index + 1);	csSss.at(index) = ss;	break;
			}
		}

		void setShaderResource(ShaderStageFlag stage, uint index, Microsoft::WRL::ComPtr<ID3D11ShaderResourceView> sr)
		{
			switch (stage)
			{
			case ShaderStageFlag::Vertex:	if (vsSrs.size() <= index) vsSrs.resize(index + 1);	vsSrs.at(index) = sr;	break;
			case ShaderStageFlag::Hull:		if (hsSrs.size() <= index) hsSrs.resize(index + 1);	hsSrs.at(index) = sr;	break;
			case ShaderStageFlag::Domain:	if (dsSrs.size() <= index) dsSrs.resize(index + 1);	dsSrs.at(index) = sr;	break;
			case ShaderStageFlag::Geometry:	if (gsSrs.size() <= index) gsSrs.resize(index + 1);	gsSrs.at(index) = sr;	break;
			case ShaderStageFlag::Pixel:	if (psSrs.size() <= index) psSrs.resize(index + 1);	psSrs.at(index) = sr;	break;
			case ShaderStageFlag::Compute:	if (csSrs.size() <= index) csSrs.resize(index + 1);	csSrs.at(index) = sr;	break;
			}
		}

		void apply(Microsoft::WRL::ComPtr<ID3D11DeviceContext> context);

	GG_ENDCLASS

}} // namespace Egg::Mesh
