#include "stdafx.h"
#include "Game.h"
#include "ThrowOnFail.h"

struct float3 {
	float x, y, z;
	float3(float x, float y, float z) :x(x), y(y), z(z) {}
};

Game::Game(Microsoft::WRL::ComPtr<ID3D11Device> device) : Egg::App(device)
{
}

Game::~Game(void)
{
}

HRESULT Game::createResources() {
	using namespace Microsoft::WRL;

	D3D11_BUFFER_DESC desc;
	desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	desc.ByteWidth = sizeof(float3) * 3;
	desc.CPUAccessFlags = 0;
	desc.MiscFlags = 0;
	desc.StructureByteStride = sizeof(float3);
	desc.Usage = D3D11_USAGE_IMMUTABLE;

	float3 vertexPositionArray[3] = {
		float3(0, 0, 0.5),
		float3(0, 1, 0.5),
		float3(1, 0, 0.5) };
	D3D11_SUBRESOURCE_DATA initData;
	initData.pSysMem = vertexPositionArray;
	initData.SysMemPitch = 0;
	initData.SysMemSlicePitch = 0;

	Egg::ThrowOnFail("Failed to create vertex buffer.", __FILE__, __LINE__) ^
		device->CreateBuffer(&desc, &initData, vertexBuffer.GetAddressOf());

	ComPtr<ID3DBlob> vertexShaderByteCode = loadShaderCode("vsIdle.cso");
	Egg::ThrowOnFail("Failed to create vertex shader.", __FILE__, __LINE__) ^
		device->CreateVertexShader(
			vertexShaderByteCode->GetBufferPointer(), vertexShaderByteCode->GetBufferSize(), NULL, vertexShader.GetAddressOf());

	// insert after #2.7
	D3D11_INPUT_ELEMENT_DESC positionElement;
	positionElement.AlignedByteOffset = 0;
	positionElement.Format = DXGI_FORMAT_R32G32B32_FLOAT;
	positionElement.InputSlot = 0;
	positionElement.InputSlotClass = D3D11_INPUT_PER_VERTEX_DATA;
	positionElement.InstanceDataStepRate = 0;
	positionElement.SemanticIndex = 0;
	positionElement.SemanticName = "POSITION";

	Egg::ThrowOnFail("Failed to create input layout.", __FILE__, __LINE__) ^
		device->CreateInputLayout(
			&positionElement, 1, vertexShaderByteCode
			->GetBufferPointer(), vertexShaderByteCode
			->GetBufferSize(), inputLayout.GetAddressOf());

	ComPtr<ID3DBlob> pixelShaderByteCode = loadShaderCode("psIdle.cso");
	Egg::ThrowOnFail("Failed to create pixel shader.", __FILE__, __LINE__) ^
		device->CreatePixelShader(
			pixelShaderByteCode->GetBufferPointer(), pixelShaderByteCode->GetBufferSize(), NULL, pixelShader.GetAddressOf());

	return S_OK;
}

HRESULT Game::releaseResources()
{
	vertexBuffer.Reset();
	inputLayout.Reset();
	vertexShader.Reset();
	pixelShader.Reset();

	return Egg::App::releaseResources();
}

void Game::render(Microsoft::WRL::ComPtr<ID3D11DeviceContext> context)
{
	context->OMSetRenderTargets(1, defaultRenderTargetView.GetAddressOf(), defaultDepthStencilView.Get());

	float clearColor[4] =
	{ 0.9f, 0.7f, 0.1f, 0.0f };
	context->ClearRenderTargetView(
		defaultRenderTargetView.Get(), clearColor);
	context->ClearDepthStencilView(defaultDepthStencilView.Get(), D3D11_CLEAR_DEPTH, 1.0, 0);

	unsigned int stride = sizeof(float3);
	unsigned int offset = 0;
	context->IASetVertexBuffers(0, 1, vertexBuffer.GetAddressOf(), &stride, &offset);
	context->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
	context->IASetInputLayout(inputLayout.Get());
	context->VSSetShader(vertexShader.Get(),
		NULL, 0);
	context->PSSetShader(pixelShader.Get(), NULL, 0);

	context->Draw(3, 0);
}
