#include "stdafx.h"
#include "Game.h"
#include "ThrowOnFail.h"
#include "Mesh/Importer.h"
#include <assimp/importer.hpp>      // C++ importer interface
#include <assimp/scene.h>       // Output data structure
#include <assimp/postProcess.h> // Post processing flags

struct float3 {
	float x, y, z;
	float3(float x, float y, float z) :x(x), y(y), z(z) {}
};

Game::Game(Microsoft::WRL::ComPtr<ID3D11Device> device) : Egg::App(device)
{
}

Game::~Game(void)
{
}

HRESULT Game::createResources() {
	using namespace Microsoft::WRL;

	inputBinder = Egg::Mesh::InputBinder::create(device);

	Assimp::Importer importer;
	const aiScene* assScene = importer.ReadFile(
		App::getSystemEnvironment().
		resolveMediaPath("giraffe.obj")
		, 0);

	Egg::Mesh::Geometry::P geometry = Egg::Mesh::Importer::fromAiMesh(device, assScene->mMeshes[0]);

	ComPtr<ID3D11VertexShader> vertexShader;
	ComPtr<ID3DBlob> vertexShaderByteCode = loadShaderCode("vsIdle.cso");
	Egg::ThrowOnFail("Failed to create vertex shader.", __FILE__, __LINE__) ^
		device->CreateVertexShader(
			vertexShaderByteCode->GetBufferPointer(), vertexShaderByteCode->GetBufferSize(), NULL, vertexShader.GetAddressOf());

	ComPtr<ID3D11PixelShader> pixelShader;
	ComPtr<ID3DBlob> pixelShaderByteCode = loadShaderCode("psIdle.cso");
	Egg::ThrowOnFail("Failed to create pixel shader.", __FILE__, __LINE__) ^
		device->CreatePixelShader(
			pixelShaderByteCode->GetBufferPointer(), pixelShaderByteCode->GetBufferSize(), NULL, pixelShader.GetAddressOf());

	Egg::Mesh::Material::P material = Egg::Mesh::Material::create(
		vertexShader,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		pixelShader,
		nullptr,
		nullptr,
		nullptr
		);
	ComPtr<ID3D11InputLayout> inputLayout = inputBinder->getCompatibleInputLayout(vertexShaderByteCode, geometry);
	shadedMesh = Egg::Mesh::Shaded::create(geometry, material, inputLayout);
	return S_OK;
}

HRESULT Game::releaseResources()
{
	return Egg::App::releaseResources();
}

void Game::render(Microsoft::WRL::ComPtr<ID3D11DeviceContext> context)
{
	context->OMSetRenderTargets(1, defaultRenderTargetView.GetAddressOf(), defaultDepthStencilView.Get());

	float clearColor[4] =
	{ 0.9f, 0.7f, 0.1f, 0.0f };
	context->ClearRenderTargetView(
		defaultRenderTargetView.Get(), clearColor);
	context->ClearDepthStencilView(defaultDepthStencilView.Get(), D3D11_CLEAR_DEPTH, 1.0, 0);

	shadedMesh->draw(context);
}
